import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

import 'mutation.dart';
import 'resolver.dart';
import 'state_container.dart';

abstract class Async<T> {}

class Uninitialized<T> extends Async<T> {}

class Loading<T> extends Async<T> {}

class Loaded<T> extends Async<T> {
  final T state;
  Loaded(this.state);
}

class AsyncResolverController<T> extends ResolverController<Async<T>> {
  AsyncResolverController(ValueNotifier<Async<T>> stateNotifier)
      : super(stateNotifier);

  void load(Future<T> Function() executor) {
    run(() async* {
      yield Loading<T>();
      final d = await executor();
      yield Loaded(d);
    });
  }
}

typedef AsyncResolverCallback<R, S, U> = Async<R> Function(
  AsyncResolverController<S> ctrl,
  Async<S> value,
  U? arg,
);

class AsyncMutationResolver<T extends Mutation, K>
    extends MutationResolver<T, Async<K>> {
  AsyncMutationResolver(
      AsyncResolverController<K> ctrl, AsyncResolverCallback<K, K, T> callback)
      : super(ctrl, (a, b, c) {
          return callback(a as AsyncResolverController<K>, b, c);
        });
}

class AsyncStateContainer<T> extends StateContainer<Async<T>> {
  AsyncStateContainer(Async<T> state) : super(state);

  @override
  AsyncResolverController<T> get resolverController =>
      AsyncResolverController(stateNotifier);

  @override
  void mutation<K extends Mutation>(AsyncResolverCallback<T, T, K> callback) {
    final mutationResolver =
        AsyncMutationResolver<K, T>(resolverController, callback);
    mutationResolvers[K] = mutationResolver;
  }
}
