import 'package:flutter/widgets.dart';

import 'state_container.dart';
import 'extensions.dart';

final Map<Widget, StateContainer> _containersMap = {};

class StateContainerSlot<T> extends StatelessWidget {
  final Widget child;
  final StateContainer<T> container;

  const StateContainerSlot({
    Key? key,
    required this.container,
    required this.child,
  }) : super(key: key);

  StatelessElement createElement() {
    return StateContainerSlotElement(this);
  }

  @override
  Widget build(BuildContext context) {
    final c = _containersMap[this] ??= container;
    return child.withContainer<T>(context, c as StateContainer<T>);
  }
}

class StateContainerSlotElement<T> extends StatelessElement {
  StateContainerSlotElement(StatelessWidget widget) : super(widget);

  @override
  void update(covariant StatelessWidget newWidget) {
    final c = _containersMap.remove(widget)!;
    _containersMap[newWidget] = c;

    super.update(newWidget);
  }

  @override
  void unmount() {
    _containersMap.remove(widget);
    super.unmount();
  }
}
